%% init
clear, close all
Ts = 1;

%% model
n = [1 1 1];
d = [1, 0.5, 1];
G = tf(n, d);
[A, B, C, D] = tf2ss(n, d);
sysc = ss(A, B, C, D);
sysd = c2d(sysc, Ts);

%% Controller
Kp = 0.6;
Ti = 1.71;

s   = tf('s');
Gr  = Kp*(1 + 1/(Ti*s));
[nc, dc] = tfdata(Gr, 'v');
[Ar, Br, Cr, Dr] = tf2ss(nc, dc);
syscPID = ss(Ar, Br, Cr, Dr);
sysdPID = c2d(syscPID, Ts);

%% Combine model Open LOOP
AOL = cell(2, 2);
AOL{1, 1} = Ar;
AOL{1, 2} = zeros(1, 2);
AOL{2, 1} = B*Cr;
AOL{2, 2} = A;
AOL = cell2mat(AOL);

BOL = [Br; B*Dr];

COLy = [D*Cr, C];
COLu = [Cr, zeros(size(C))];

DOLy = D*Dr;
DOLu = Dr;

syscOL = ss(AOL, BOL, [COLu; COLy], [DOLu; DOLy]);

%% Combine model Closed LOOP
ID = eye(size(DOLy));
DD1 = inv(ID + DOLy);
ACL = AOL - BOL*COLy*DD1;
BCL = BOL - BOL*DOLy*DD1;
CCLy = COLy*DD1;
DCLy = DOLy*DD1;

CCLu = COLu - DOLu*COLy*DD1;
DCLu = DOLu - DOLu*DOLy*DD1;

syscCL = ss(ACL, BCL, [CCLu; CCLy], [DCLu; DCLy]);

%% sun simulink
st = 1;
iv = 0;
fv = 1;

time = 20;

sim('SISO_closed_loop', [0, time])