%% init
clear, close all

%% SS model
A = [-2.509e-3,  0  , 0;
       6.94e-2, -0.1, 0;
      -6.69e-3, 0, 0];

B = [0.9, -0.349, -0.15;
    0, 14.155, 0;
    0, -1.389, 1.569];

C = [1, 0, 0;
    0, 1, 0;
    6.34e-3, 0, 4.71e-3];

D = zeros(3, 3);
D(3, :) = [0.253, 0.512, -0.014];

syscBOILER = ss(A, B, C, D);

%% Controller
kp1 = 11.1185;
kp2 = 0.004;
kp3 = 1.1631;
ki1 = 0.0033;
ki2 = 0.0093;
ki3 = 0.0186;
k12 = 0.0292;
k13 = 0.1344;
k21 = 0.0468;
k23 = 0.0875;
k31 = 0.0842;
k32 = 0.0699;

s = tf('s');
G1 = kp1 + ki1/s;
[cn1, cd1] = tfdata(G1, 'v');

G2 = kp2 + ki2/s;
[cn2, cd2] = tfdata(G2, 'v');

G3 = kp3 + ki3/s;
[cn3, cd3] = tfdata(G3, 'v');

[Ar1, Br1, Cr1, Dr1] = tf2ss(cn1, cd1);
[Ar2, Br2, Cr2, Dr2] = tf2ss(cn2, cd2);
[Ar3, Br3, Cr3, Dr3] = tf2ss(cn3, cd3);

Ar = blkdiag(Ar1, Ar2, Ar3);
Br = blkdiag(Br1, Br2, Br3);
Cr = blkdiag(Cr1, Cr2, Cr3);
Dr = blkdiag(Dr1, Dr2, Dr3);
Dr(1, 2) = k21;
Dr(1, 3) = k31;
Dr(2, 1) = k12;
Dr(2, 3) = k31;
Dr(3, 1) = k13;
Dr(3, 2) = k23;

syscPID = ss(Ar, Br, Cr, Dr);

%% Open lopp model
AA = cell(2, 2);
AA{1, 1} = Ar;
AA{1, 2} = zeros(size(Ar, 1), size(B*Cr, 2));
AA{2, 1} = B*Cr;
AA{2, 2} = A;
AA  = cell2mat(AA);
BB  = [Br; B*Dr];
CC  = [D*Cr, C];
DD  = D*Dr;
CCu = [Cr, zeros(size(Cr, 1), size(Cr, 2)); D*Cr, C];
DDu = [Dr; D*Dr];

syscOL = ss(AA, BB, CCu, DDu);

%% Closed Loop Model
ID = eye(size(D));
D1 = (ID + DD);
ACL = AA - BB*D1*CC;
BCL = BB - BB*D1*DD;
CCLy = D1*CC;
DCLy = D1*DD;
CCLu = [Cr, Dr];

syscCL = ss(ACL, BCL, [CCLy*0; CCLy], [DCLy*0; DCLy]);

%% run simulation
power_step_time = 20;

st_lti = [0, power_step_time, 0];
iv_lti = [0, 0, 0];
fv_lti = [0.0, 2, 0.0];
u0 = [0, 0, 0];
x0 = [108, 66.65, 428];

qe  = (0.854*u0(2) - 0.147)*x0(1) + 15.59*u0(1) - 2.514*u0(3) - 2.096;
acs = 1;
Xw0 = 0.05*(0.13073*x0(3) + 100*acs + qe/9 - 67.975);

y0 = [x0(1), x0(2), Xw0];

sim('baseline_model_pid', [0, 300])