%% init
clear, close all

%% load data
mpc_simple = load('mpc_results_simple');
mpc_pid    = load('mpc_results_pid');
y0 = mpc_simple.y0;
dumin = mpc_simple.dumin;
dumax = mpc_simple.dumax;
Ts    = mpc_simple.Ts;

%% display
y_labels = {'Pressure', 'Power', 'Water'};
figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on
    stairs(mpc_simple.tt, mpc_simple.Rplot(k, :) + y0(k), 'g')
    plot(mpc_simple.y_pid(:, 1), mpc_simple.y_pid(:, k+1), 'm')
    stairs(mpc_simple.tt, mpc_simple.y(k, :), 'c')
    stairs(mpc_pid.tt, mpc_pid.y(k, :), 'k')
    hold off
    ylabel(y_labels{k})
    if k == 1
        leg = legend('Ref', 'PID', 'MPC', 'MPC-PID');
        set(leg, 'orientation', 'horizontal')
    end
    if k == 3
        xlabel('Time [s]')
    end
end
fs('Outputs')

figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on
%     stairs(mpc_simple.tt, mpc_simple.Rplot(k, :) + y0(k), 'g')
    plot(mpc_simple.u_pid(:, 1), mpc_simple.u_pid(:, k+1), 'm')
    stairs(mpc_simple.tt, mpc_simple.u(k, :), 'c')
%     stairs(mpc_pid.tt, mpc_pid.y(k, :), 'k')
    hold off
    ylabel(y_labels{k})
    if k == 1
        leg = legend('PID', 'MPC');
        set(leg, 'orientation', 'horizontal')
    end
    if k == 3
        xlabel('Time [s]')
    end
end
fs('Inputs')

y_labels = {'Fuel', 'Steam', 'Feedwater'};
figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on
    stairs(mpc_simple.u_pid(1:end-1, 1), diff(mpc_simple.u_pid(:, k+1))/Ts, 'm')
    stairs(mpc_simple.tt(1:end-1), diff(mpc_simple.u(k, :))/Ts, 'c')
    if k ~= 2
        stairs(mpc_simple.tt(1:end-1), diff(mpc_simple.u(k, :))*0+dumin(k)/Ts, 'r')
    end
    stairs(mpc_simple.tt(1:end-1), diff(mpc_simple.u(k, :))*0+dumax(k)/Ts, 'r')
    hold off
    ylabel(y_labels{k})
    if k == 1
        leg = legend('PID', 'MPC');
        set(leg, 'orientation', 'horizontal')
    end
    if k == 3
        xlabel('Time [s]')
    end
end
fs('dInputs')

y_labels = {'Pressure', 'Power', 'Water'};
figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on
    stairs(mpc_simple.tt, mpc_simple.Rplot(k, :), 'g')
    stairs(mpc_pid.tt, mpc_pid.u(k, :), 'k')
    hold off
    ylabel(y_labels{k})
    if k == 1
        leg = legend('Ref', 'PID setpoint');
        set(leg, 'orientation', 'horizontal')
    end
    if k == 3
        xlabel('Time [s]')
    end
end
fs('PID_setpoints')