%% init
clear, close all

%% state space model
A = [-2.509e-3,  0  , 0;
       6.94e-2, -0.1, 0;
      -6.69e-3, 0, 0];

B = [0.9, -0.349, -0.15;
    0, 14.155, 0;
    0, -1.389, 1.569];

C = [1, 0, 0;
    0, 1, 0;
    6.34e-3, 0, 4.71e-3];

D = zeros(3, 3);
D(3, :) = [0.253, 0.512, -0.014];

Ts = 2;
sysc = ss(A, B, C, D);
sysd = c2d(sysc, Ts);

%% Setup simulation
% st -> step time
% iv -> initial value
% fv -> final value

st_lti = [0, 20, 0];
iv_lti = [0, 0, 0];
fv_lti = [0.0, 0.1, 0.0];
time = 400;
u0 = [0, 0, 0];
x0 = [108, 66.65, 428];

qe  = (0.854*u0(2) - 0.147)*x0(1) + 15.59*u0(1) - 2.514*u0(3) - 2.096;
acs = 1;
Xw0 = 0.05*(0.13073*x0(3) + 100*acs + qe/9 - 67.975);

y0 = [x0(1), x0(2), Xw0];

save('data_model', 'sysd', 'sysc', 'Ts', 'u0', 'y0', 'x0', 'Xw0')

sim('baseline_model', [0 time])

%% D-time simulation setup
Tmax = time/60; % in "min"
kf   = Tmax*60/Ts;
t    = 0:Ts:kf;
tt   = linspace(0, Tmax*60, kf);
x    = zeros(3, kf);
y    = zeros(3, kf);
u    = zeros(3, kf);
u(2, 20/Ts+1:end) = fv_lti(2);

%% run simulation
tic;
display('----------------------------')
display('Entering the simulation loop')
fprintf(1,'Status:     ');
for k = 1:1:kf
    %for-loop diagnostic
    for_diag(k, kf)

    x(:, k+1) = sysd.A*x(:, k) + sysd.B*u(:, k);
    y(:, k)   = sysd.C*x(:, k) + sysd.D*u(:, k) + y0';      
end
fprintf('\n')
time = toc;
cprintf([0.0, 0.0, 0.8], 'Simulation complete\n');
display(['Elapsed time: ' num2str(time) ' s'])
display(['Samples per second: ' num2str(kf/time)])
display('Displaying results')

%% display
close all
figure
hold on
stairs(tt, y(1, :), 'r');
plot(y_lti(:, 1), y_lti(:, 2), 'b')
ylabel('Pressure [$kg/cm^2$]')
xlabel('Time [s]')
hold off
fig2pdf('pressure_step')

figure
hold on
stairs(tt, y(2, :), 'r');
plot(y_lti(:, 1), y_lti(:, 3), 'b')
ylabel('Power [MW]')
xlabel('Time [s]')
hold off
fig2pdf('power_step')

figure
hold on
stairs(tt, y(3, :), 'r');
plot(y_lti(:, 1), y_lti(:, 4), 'b')
ylabel('Water level [m]')
xlabel('Time [s]')
hold off
fig2pdf('water_step')

