%% init
clear, close all

%% load model
load('data_model');
ss_m.A = sysd.A;
ss_m.B = sysd.B;
ss_m.C = sysd.C;
ss_m.D = sysd.D;

ss_m.E = ss_m.B*0;
ss_m.F = ss_m.D*0;

weight.Q  = diag([0.9, 0.5, 1.5]);
weight.S  = diag([1, 100, 1]);
weight.Sy = diag([1, 1, 1]);
weight.Su = diag([1, 1, 1]);

N = 10;

mpc_matrix = mpc_matrix_design(ss_m, weight, N);
H = mpc_matrix.H;
MX = mpc_matrix.Mx;
MR = mpc_matrix.Mr;
MU = mpc_matrix.Mu;

time = 600;
power_step_time = 100;
run run_simulation
sim('baseline_model_pid', [0, time]);

%% setup simulation
Tmax = time/60; % in "min"
kf   = Tmax*60/Ts;
t    = 0:Ts:kf;
tt   = linspace(0, Tmax*60, kf);

x = zeros(size(ss_m.A, 1), kf);
y = zeros(3, kf);
u = zeros(3, kf);
u_qp = [0, 0, 0]';
nu = 3;
reference.init  = [0; 0; 0];
reference.value = [0; 5; 0];
reference.start = [0; power_step_time/Ts+1; 0];
reference.end   = [kf+N-1; kf+N-1; kf+N-1];
[R, Rstairs] = reference_build(N, kf, reference);

%% run simulation
tic;
display('----------------------------')
display('Entering the simulation loop')
fprintf(1,'Status:     ');
for k = 1:1:kf
    % for-loop diagnostic
    for_diag(k, kf)
    
    % MPC controller
    dR = cell2mat(R(k:k+N-1));
    
    g  = MX*x(:, k) + MR*dR + MU*u_qp;
    U = qpas(H, g, [], [], [], [], [], []);

    u_qp = U(1:nu);
    u_in = u_qp;
    
    x(:, k+1) = ss_m.A*x(:, k) + ss_m.B*u_in;
    y(:, k)   = ss_m.C*x(:, k) + ss_m.D*u_in + y0';
    u(:, k) = u_in;
    
end
fprintf('\n')
time = toc;
cprintf([0.0, 0.0, 0.8], 'Simulation complete\n');
display(['Elapsed time: ' num2str(time) ' s'])
display(['Samples per second: ' num2str(kf/time)])

%% Display results
display('Displaying results')
figure
subplot(3, 1, 1)
    hold on
    stairs(tt, Rstairs(1, :) + y0(1), 'r')
    stairs(tt, y(1, :))
    hold off
    ylabel('Pressure')
subplot(3, 1, 2)
    hold on
    stairs(tt, Rstairs(2, :) + y0(2), 'r')
    stairs(tt, y(2, :))
    hold off
    ylabel('Power')
subplot(3, 1, 3)
    hold on
    stairs(tt, Rstairs(3, :) + y0(3), 'r')
    stairs(tt, y(3, :))
    hold off
    ylabel('Xw')

figure
hold on
stairs(tt, u(1, :), 'b')
stairs(tt, u(2, :), 'r')
stairs(tt, u(3, :), 'm')
hold off