%% init
clear, close all

%% load
% nominal_models = load('nominal_models');
% syscBOILER = nominal_models.syscBOILER;

run boiler_cl_model_rate

%% slew rates
rate_u1 = [0.007; -0.007];
rate_u2 = [0.02; -2];
rate_u3 = [0.05; -0.05];
dumin = [-0.007; -2; -0.05];
dumax = [0.007; 0.02; 0.05];

sim('baseline_model_pid_rate', [0 500])
time = y_pid(:, 1);
%% labels and other data
fpath     = '../thesis/images/pid_rate/';
papersize = [6, 5]*1.5;
y_labels  = {'$p\;[MPa]$', '$P_0\;[MW]$', '$X_W\;[m]$'};
u_labels = {'$u_F\;[\%]$', '$u_S\;[\%]$', '$u_W\;[\%]$'};
du_labels = {'$\triangle u_F\;[\%/s]$', '$\triangle u_S\;[\%/s]$', ...
             '$\triangle u_W\;[\%/s]$'};
save_y    = {'y1', 'y2', 'y3'};
save_u    = {'u1', 'u2', 'u3'};
save_du   = {'du1', 'du2', 'du3'};
kgcm2_to_pa = 98066.5e-6;


%% display Outputs
close all
for k = 1:1:3
    figure(k)
    hold on
    if k == 1
        plot(time, (y_pid(:, k+1) + ys(k)).*kgcm2_to_pa, 'b')
        stairs(time, (r_pid(:, k+1) + ys(k)).*kgcm2_to_pa, 'g')
    else
        stairs(time, y_pid(:, k+1) + ys(k), 'b')
        stairs(time, (r_pid(:, k+1) + ys(k)), 'g')
    end
    hold off
    ylabel(y_labels{k})
    leg = legend('y', 'r', 2);
%     if k == 1
%         leg = legend('Ref', 'PID', 'MPC', 'MPC-PID');
%     end
    xlabel('Time [s]')
    set(gca, 'Xlim', [0 time(end)]);
    fsp(save_y{k}, save_y{k}, fpath, papersize)
end

%% display Inputs
close all
for k = 1:1:3
    figure(k)
    hold on
    plot(u_pid(:, 1), u_pid(:, k+1) + us(k))
    stairs(time, time*0 + 0, 'r')
    stairs(time, time*0 + 1, 'r')
    hold off
    ylabel(u_labels{k})
%     leg = legend('y', 'r');
%     set(leg, 'orientation', 'horizontal')
    xlabel('Time [s]')
%     set(gca, 'Xlim', [0 time(end)], 'Ylim', [-0.05, 1.05], 'Ytick', 0:0.2:1);
    fsp(save_u{k}, save_u{k}, fpath, papersize)
end

%% display dInputs
close all
for k = 1:1:3
    figure(k)
    hold on
    plot(time(1:end-1), diff(u_pid(:, k+1)))
    stairs(time, time*0 + dumin(k), 'r')
    stairs(time, time*0 + dumax(k), 'r')
    hold off
    ylabel(du_labels{k})
%     leg = legend('PID', 'MPC', 'MPC-PID');
%     set(leg, 'orientation', 'horizontal')
    xlabel('Time [s]')
%     set(gca, 'Xlim', [0 time(end)], 'Ylim', [-0.05, 1.05], 'Ytick', 0:0.2:1);
    fsp(save_u{k}, save_du{k}, fpath, papersize)
end

% figure
% for k = 1:1:3
%     subplot(3, 1, k)
%     hold on
%     plot(y_pid(:, 1), y_pid(:, k+1) + ys(k))
%     plot(r_pid(:, 1), r_pid(:, k+1) + ys(k), 'g')
%     hold off
% end
% fsp('outputs')
% 
% figure
% for k = 1:1:3
%     subplot(3, 1, k)
%     hold on
%     plot(u_pid(:, 1), u_pid(:, k+1) + us(k))
%     hold off
% end
% fsp('Inputs')
% 
% figure
% for k = 1:1:3
%     subplot(3, 1, k)
%     hold on
%     plot(u_pid(1:end-1, 1), diff(u_pid(:, k+1)))
%     hold off
% end
% fsp('dInputs')