%%init
clear, close all

power_step_time = 20;
%% state space model
A = [-2.509e-3,  0  , 0;
       6.94e-2, -0.1, 0;
      -6.69e-3, 0, 0];

B = [0.9, -0.349, -0.15;
    0, 14.155, 0;
    0, -1.389, 1.659];

C = [1, 0, 0;
    0, 1, 0;
    6.34e-3, 0, 4.71e-3];

D = zeros(3, 3);
D(3, :) = [0.253, 0.512, -0.014];
syscBOILER = ss(A, B, C, D);

st_lti = [0, power_step_time, 0];
iv_lti = [0, 0, 0];
fv_lti = [0.0, 5, 0.0];
u0 = [0, 0, 0];
us = [0.34; 0.69; 0.436];
x0 = [108, 66.65, 428];
xs = x0;

qe  = (0.854*us(2) - 0.147)*x0(1) + 45.59*us(1) - 2.514*us(3) - 2.096;
% acs = 1;
acs =   ( (1-0.001538*x0(3)) * (0.8*x0(1) - 25.6) ) /...
        ( x0(3)*(1.0394 - 0.0012304*x0(1)) );

Xw0 = 0.05*(0.13073*x0(3) + 100*acs + qe/9 - 67.975);


y0 = [x0(1), x0(2), Xw0];
ys = [x0(1), x0(2), Xw0];
us = [0.34; 0.69; 0.436];

save('yus', 'y0', 'us', 'ys', 'xs');

y0 = [x0(1), x0(2), Xw0]*0;
%% Controller
kp1 = 11.1185;
kp2 = 0.004;
kp3 = 1.1631;
ki1 = 0.0033;
ki2 = 0.0093;
ki3 = 0.0186;
k12 = 0.0292;
k13 = 0.1344;
k21 = 0.0468;
k23 = 0.0875;
k31 = 0.0842;
k32 = 0.0699;

s = tf('s');
G1 = kp1 + ki1/s;
[cn1, cd1] = tfdata(G1, 'v');

G2 = kp2 + ki2/s;
[cn2, cd2] = tfdata(G2, 'v');

G3 = kp3 + ki3/s;
[cn3, cd3] = tfdata(G3, 'v');

[Ar1, Br1, Cr1, Dr1] = tf2ss(cn1, cd1);
[Ar2, Br2, Cr2, Dr2] = tf2ss(cn2, cd2);
[Ar3, Br3, Cr3, Dr3] = tf2ss(cn3, cd3);

Ar = blkdiag(Ar1, Ar2, Ar3);
Br = blkdiag(Br1, Br2, Br3);
Cr = blkdiag(Cr1, Cr2, Cr3);
Dr = blkdiag(Dr1, Dr2, Dr3);
Dr(1, 2) = k21;
Dr(1, 3) = k31;
Dr(2, 1) = k12;
Dr(2, 3) = k31;
Dr(3, 1) = k13;
Dr(3, 2) = k23;

syscPID = ss(Ar, Br, Cr, Dr);

%% Openloop model
AOL = cell(2, 2);
AOL{1, 1} = Ar;
AOL{1, 2} = zeros(size(Ar, 1), size(B*Cr, 2));
AOL{2, 1} = B*Cr;
AOL{2, 2} = A;
AOL = cell2mat(AOL);

BOL = [Br; B*Dr];

COLy = [D*Cr, C];
COLu = [Cr, zeros(size(C))];

DOLy = D*Dr;
DOLu = Dr;

syscOL = ss(AOL, BOL, [COLu; COLy], [DOLu; DOLy]);

%% Combine model Closed LOOP
ID = eye(size(DOLy));
DD1 = inv(ID + DOLy);
ACL = AOL - BOL*DD1*COLy;
BCL = BOL - BOL*DD1*DOLy;
CCLy = DD1*COLy;
DCLy = DD1*DOLy;

CCLu = COLu - DOLu*DD1*COLy;
DCLu = DOLu - DOLu*DD1*DOLy;

syscCL = ss(ACL, BCL, [CCLu; CCLy], [DCLu; DCLy]);
Ts1 = 0.5;
Ts2 = 0.1;
sysdCL1 = c2d(syscCL, Ts1);
sysdCL2 = c2d(syscCL, Ts2);

step(syscCL, sysdCL1, sysdCL2, 10)

save('nominal_models', 'syscCL', 'syscOL', 'syscBOILER', 'syscPID', 'us', 'ys', 'xs')
%% simulate model
% sim('baseline_model_pid', [0 100])
% sim('baseline_model_pid_rate', [0 100])

%% tf
for k = 1:1:3
    [n, d] = ss2tf(ACL, BCL, CCLy, DCLy, k);
    for j = 1:1:3
        G{j, k} = minreal(tf(n(j, :), d));
    end
end
rd = min((-1)./real(roots(d)));
