%% init
clear, close all

%% load data
load('nominal_models.mat')
load('yus.mat')
fpath = '../thesis/images/dtime';

%% diskretization
kp1 = 11.1185;
kp2 = 0.004;
kp3 = 1.1631;
ki1 = 0.0033;
ki2 = 0.0093;
ki3 = 0.0186;
k12 = 0.0292;
k13 = 0.1344;
k21 = 0.0468;
k23 = 0.0875;
k31 = 0.0842;
k32 = 0.0699;

s = tf('s');
G1 = kp1 + ki1/s;
[cn1, cd1] = tfdata(G1, 'v');

G2 = kp2 + ki2/s;
[cn2, cd2] = tfdata(G2, 'v');

G3 = kp3 + ki3/s;
[cn3, cd3] = tfdata(G3, 'v');



%% BT system system
papersize = [7, 6]*2;
Ts1 = 10;
Ts2 = 5;
Ts3 = 0.5;
ss_m1 = c2d(syscBOILER, Ts1);
ss_m2 = c2d(syscBOILER, Ts2);
ss_m3 = c2d(syscBOILER, Ts3);

st_lti = [0, 10, 0];
iv_lti = [0, 0, 0];
fv_lti = [0, 1, 0];
sim('baseline_model_dtime', [0 100])
figure
hold on
stairs(y_cl1(:, 1), y_cl1(:, 3), 'r')
stairs(y_cl2(:, 1), y_cl2(:, 3), 'k')
stairs(y_cl3(:, 1), y_cl3(:, 3), 'c')
plot(y_pid(:, 1), y_pid(:, 3), 'b')
hold off
xlabel('Time [s]')
ylabel('$P_0 [MW]$')
legend('$T_s = 10s$', '$T_s = 5s$', '$T_s = 0.5s$', 'Continuous', 4)
title('\bf Boiler-turbine system')

fsp('model_dtime_cmp', 'model_dtime_cmp', fpath, papersize)


%% Closed loop system
Ts1 = 1;
Ts2 = 0.5;
Ts3 = 0.1;
sysdCL1 = c2d(syscCL, Ts1);
sysdCL2 = c2d(syscCL, Ts2);
sysdCL3 = c2d(syscCL, Ts3);

st_lti = [1, 0, 0];
iv_lti = [0, 0, 0];
fv_lti = [1.0, 0.0, 0.0];
sim('baseline_model_pid_dtime', [0 3])
figure
hold on
for k = 1:1:3
    stairs(y_cl1(:, 1), y_cl1(:, 2), 'r')
    stairs(y_cl2(:, 1), y_cl2(:, 2), 'k')
    stairs(y_cl3(:, 1), y_cl3(:, 2), 'c')
    plot(u_pid(:, 1), u_pid(:, 2), 'b')
end
title('\bf Closed loop system')
hold off
xlabel('Time [s]')
ylabel('$u_F [\%]$')
legend('$T_s = 1s$', '$T_s = 0.5s$', '$T_s = 0.1s$', 'Continuous', 4)

fsp('model_pid_dtime_cmp', 'model_pid_dtime_cmp', fpath, papersize)

%% PZ map bt system
papersize = [6, 6]*2;
figure
hold on
[p, z] = pzmap(ss_m1);
    plot(real(p), imag(p), 'bx', 'markersize', 10)
[p, z] = pzmap(ss_m2);
    plot(real(p), imag(p), 'gx', 'markersize', 10)
[p, z] = pzmap(ss_m3);
    plot(real(p), imag(p), 'rx', 'markersize', 10)
[p, z] = pzmap(ss_m1);
    plot(real(z), imag(z), 'bo', 'markersize', 10)
[p, z] = pzmap(ss_m2);
    plot(real(z), imag(z), 'go', 'markersize', 10)
[p, z] = pzmap(ss_m3);
    plot(real(z), imag(z), 'ro', 'markersize', 10)
plot([-1 1], [0 0], 'Linestyle', '--', 'color', [0.6, 0.6, 0.6])
plot([0 0], [-1 1], 'Linestyle', '--', 'color', [0.6, 0.6, 0.6])
t = 0:0.001:2*pi;
plot(sin(t), cos(t), 'Linestyle', '--', 'color', [0.6, 0.6, 0.6])
ylabel('Imaginary')
xlabel('Real')
leg = legend('$T_s = 10s$', '$T_s = 5s$', '$T_s = 0.5s$');
set(leg, 'location', 'NorthOutside', 'orientation', 'horizontal')
title('\bf Boiler-turbine system')
    
axis square
axis([-1 1 -1 1])
grid off
fsp('pzmap_m', 'pzmap_m', fpath, papersize)

%% PZ map bt system
papersize = [6, 6]*2;
figure
hold on
[p, z] = pzmap(sysdCL1);
    plot(real(p), imag(p), 'bx', 'markersize', 10)
    plot(real(z), imag(z), 'bo', 'markersize', 10)
[p, z] = pzmap(sysdCL2);
    plot(real(p), imag(p), 'gx', 'markersize', 10)
    plot(real(z), imag(z), 'go', 'markersize', 10)
[p, z] = pzmap(sysdCL3);
    plot(real(p), imag(p), 'rx', 'markersize', 10)
    plot(real(z), imag(z), 'ro', 'markersize', 10)
plot([-1 1], [0 0], 'Linestyle', '--', 'color', [0.6, 0.6, 0.6])
plot([0 0], [-1 1], 'Linestyle', '--', 'color', [0.6, 0.6, 0.6])
t = 0:0.001:2*pi;
plot(sin(t), cos(t), 'Linestyle', '--', 'color', [0.6, 0.6, 0.6])
ylabel('Imaginary')
xlabel('Real')
leg = legend('$T_s = 1s$', '$T_s = 0.5s$', '$T_s = 0.1s$');
set(leg, 'location', 'NorthOutside', 'orientation', 'horizontal')
title('\bf Closed loop system')
axis square
axis([-1 1 -1 1])
grid off
fsp('pzmap_pid', 'pzmap_pid', fpath, papersize)