%% init
clear, close all

%% With reference
extension = '_ref';
nom_model  = load('../boiler_ss_models/nominal_models.mat');
mpc_simple = load(['mpc_results_simple' extension]);
mpc_pid    = load(['mpc_results_pid' extension]);

%% extract data
y0     = mpc_simple.y0;
dumin  = mpc_simple.dumin;
dumax  = mpc_simple.dumax;
Ts     = mpc_simple.Ts;
us     = nom_model.us;
time   = mpc_simple.tt;
time_s = mpc_simple.y_pid(:, 1);

%% data convert
kgcm2_to_pa = 98066.5e-6;
mpc_simple.y(1, :)     = mpc_simple.y(1, :).*kgcm2_to_pa;
mpc_pid.y(1+3, :)      = mpc_pid.y(1+3, :).*kgcm2_to_pa;
mpc_simple.y_pid(:, 2) = mpc_simple.y_pid(:, 2).*kgcm2_to_pa;
mpc_simple.Rplot(1, :) = (mpc_simple.Rplot(1, :) + y0(1)).*kgcm2_to_pa;
mpc_simple.Rplot(2, :) = (mpc_simple.Rplot(2, :) + y0(2)).*1;
mpc_simple.Rplot(3, :) = (mpc_simple.Rplot(3, :) + y0(3)).*1;
mpc_pid.Rplot(1+3, :) = (mpc_pid.Rplot(1+3, :) + y0(1)).*kgcm2_to_pa;
mpc_pid.Rplot(2+3, :) = (mpc_pid.Rplot(2+3, :) + y0(2)).*1;
mpc_pid.Rplot(3+3, :) = (mpc_pid.Rplot(3+3, :) + y0(3)*0).*1;
mpc_simple.r_pid(:, 2) = mpc_simple.r_pid(:, 2).*kgcm2_to_pa;

travel_1 = zeros(4, 7);
for k = 1:1:3
    travel_1(2, k+1) = sum(abs(diff(mpc_simple.u_pid(:, k+1))));
    travel_1(3, k+1) = sum(abs(diff(mpc_simple.u(k, :))/Ts));
    travel_1(4, k+1) = sum(abs(diff(mpc_pid.y(k, :))/Ts));
    travel_1(:, k+4) = travel_1(:, k+1)./travel_1(2, k+1).*100;
end

iae_1 = zeros(4, 7);
for k = 1:1:3
    iae_1(2, k+1) = sum(abs(mpc_simple.y_pid(:, k+1) - mpc_simple.r_pid(:, k+1)));
    iae_1(3, k+1) = sum(abs(mpc_simple.y(k, :) - mpc_simple.Rplot(k, :)));
    iae_1(4, k+1) = sum(abs(mpc_pid.y(k+3, :) - mpc_pid.Rplot(k+3, :)));
    iae_1(:, k+4) = iae_1(:, k+1)./iae_1(2, k+1).*100;
end

%% No reference
clear mpc_simple mpc_pid nom model
extension = '_no_ref';
nom_model  = load('../boiler_ss_models/nominal_models.mat');
mpc_simple = load(['mpc_results_simple' extension]);
mpc_pid    = load(['mpc_results_pid' extension]);

%% extract data
y0     = mpc_simple.y0;
dumin  = mpc_simple.dumin;
dumax  = mpc_simple.dumax;
Ts     = mpc_simple.Ts;
us     = nom_model.us;
time   = mpc_simple.tt;
time_s = mpc_simple.y_pid(:, 1);

%% data convert
kgcm2_to_pa = 98066.5e-6;
mpc_simple.y(1, :)     = mpc_simple.y(1, :).*kgcm2_to_pa;
mpc_pid.y(1+3, :)      = mpc_pid.y(1+3, :).*kgcm2_to_pa;
mpc_simple.y_pid(:, 2) = mpc_simple.y_pid(:, 2).*kgcm2_to_pa;
mpc_simple.Rplot(1, :) = (mpc_simple.Rplot(1, :) + y0(1)).*kgcm2_to_pa;
mpc_simple.Rplot(2, :) = (mpc_simple.Rplot(2, :) + y0(2)).*1;
mpc_simple.Rplot(3, :) = (mpc_simple.Rplot(3, :) + y0(3)).*1;
mpc_pid.Rplot(1+3, :) = (mpc_pid.Rplot(1+3, :) + y0(1)).*kgcm2_to_pa;
mpc_pid.Rplot(2+3, :) = (mpc_pid.Rplot(2+3, :) + y0(2)).*1;
mpc_pid.Rplot(3+3, :) = (mpc_pid.Rplot(3+3, :) + y0(3)).*1;
mpc_simple.r_pid(:, 2) = mpc_simple.r_pid(:, 2).*kgcm2_to_pa;

travel_2 = zeros(3, 7);
for k = 1:1:3
    travel_2(2, k+1) = sum(abs(diff(mpc_simple.u_pid(:, k+1))));
    travel_2(3, k+1) = sum(abs(diff(mpc_simple.u(k, :))/Ts));
    travel_2(4, k+1) = sum(abs(diff(mpc_pid.y(k, :))/Ts));
    travel_2(:, k+4) = travel_2(:, k+1)./travel_2(2, k+1).*100;
end

iae_2 = zeros(3, 3);
for k = 1:1:3
    iae_2(2, k+1) = sum(abs(mpc_simple.y_pid(:, k+1) - mpc_simple.r_pid(:, k+1)));
    iae_2(3, k+1) = sum(abs(mpc_simple.y(k, :) - mpc_simple.Rplot(k, :)));
    iae_2(4, k+1) = sum(abs(mpc_pid.y(k+3, :) - mpc_pid.Rplot(k+3, :)));
    iae_2(:, k+4) = iae_2(:, k+1)/iae_2(2, k+1).*100;
end

%% merge
iae = [[zeros(4, 1), iae_2]; [zeros(4, 1),iae_1]];
iae(5:6, :) = [];

travel = [[zeros(4, 1), travel_2]; [zeros(4, 1), travel_1]];
travel(5:6, :) = [];
