%% init
clear, close all

%% load data
nom_model  = load('../boiler_ss_models/nominal_models.mat');
mpc_simple = load('mpc_results_simple');
mpc_pid    = load('mpc_results_pid');

%% extract data
y0     = mpc_simple.y0;
dumin  = mpc_simple.dumin;
dumax  = mpc_simple.dumax;
Ts     = mpc_simple.Ts;
us     = nom_model.us;
time   = mpc_simple.tt;
time_s = mpc_simple.y_pid(:, 1);

%% labels and other data
fapth = './images/';
y_labels = {'$p\;[MPa]$', '$P_0\;[MW]$', '$X_W\;[m]$'};
u_labels = {'$u_1\;[\%]$', '$u_2\;[\%]$', '$u_3\;[\%]$'};
du_labels = {'$\triangle u_1\;[\%/s]$', '$\triangle u_2\;[\%/s]$', ...
             '$\triangle u_3\;[\%/s]$'};

%% data convert
kgcm2_to_pa = 98066.5e-6;
mpc_simple.y(1, :)     = mpc_simple.y(1, :).*kgcm2_to_pa;
mpc_pid.y(1+3, :)      = mpc_pid.y(1+3, :).*kgcm2_to_pa;
mpc_simple.y_pid(:, 2) = mpc_simple.y_pid(:, 2).*kgcm2_to_pa;

%% display Outputs
figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on
    if k == 1
        stairs(time, (mpc_simple.Rplot(k, :) + y0(k)).*kgcm2_to_pa, 'g')
    else
        stairs(time, mpc_simple.Rplot(k, :) + y0(k), 'g')
    end
    plot(time_s, mpc_simple.y_pid(:, k+1), 'm')
    stairs(time, mpc_simple.y(k, :), 'b')
    stairs(mpc_pid.tt, mpc_pid.y(k+3, :), 'k')
    hold off
    ylabel(y_labels{k})
    if k == 1
        leg = legend('Ref', 'PID', 'MPC', 'MPC-PID');
        set(leg, 'orientation', 'horizontal')
    end
    if k == 3
        xlabel('Time [s]')
    end
end
fs('Outputs')

%% display Inputs
figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on
%     stairs(time, mpc_simple.Rplot(k, :) + y0(k), 'g')
    plot(mpc_simple.u_pid(:, 1), mpc_simple.u_pid(:, k+1) + us(k), 'm')
    stairs(time, mpc_simple.u(k, :) + us(k), 'b')
    stairs(mpc_pid.tt, mpc_pid.y(k, :) + us(k), 'k')
    hold off
    ylabel(u_labels{k})
    if k == 1
        leg = legend('PID', 'MPC', 'MPC-PID');
        set(leg, 'orientation', 'horizontal')
    end
    if k == 3
        xlabel('Time [s]')
    end
end
fs('Inputs')

%% display delta Inputs
figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on
    stairs(mpc_simple.u_pid(1:end-1, 1), diff(mpc_simple.u_pid(:, k+1)), 'm')
    stairs(time(1:end-1), diff(mpc_simple.u(k, :))/Ts, 'b')
    stairs(mpc_pid.tt(1:end-1), diff(mpc_pid.y(k, :))/Ts, 'k')
    if k ~= 2
        stairs(time, mpc_simple.u(k, :)*0+dumin(k)/Ts, 'r')
    end
    stairs(time, mpc_simple.u(k, :)*0+dumax(k)/Ts, 'r')
    hold off
    ylabel(du_labels{k})
    if k == 1
        leg = legend('PID', 'MPC', 'MPC-PID');
        set(leg, 'orientation', 'horizontal')
    end
    if k == 3
        xlabel('Time [s]')
    end
    
    if k == 1
        axis([0 100 -0.025 0.025])
    elseif k == 2
        axis([0 100 -0.12 0.05])        
    elseif k == 3
        axis([0 100 -0.1 0.1])
    end
end
fs('dInputs')

%% display delta input travel
figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on
    stairs(mpc_simple.u_pid(1:end-1, 1), cumsum(abs(diff(mpc_simple.u_pid(:, k+1)))), 'm')
    stairs(time(1:end-1), cumsum(abs(diff(mpc_simple.u(k, :))/Ts)), 'b')
    stairs(mpc_pid.tt(1:end-1), cumsum(abs(diff(mpc_pid.y(k, :))/Ts)), 'k')
    ylabel(u_labels{k})
    if k == 1
        leg = legend('PID', 'MPC', 'MPC-PID');
        set(leg, 'orientation', 'horizontal')
    end
    if k == 3
        xlabel('Time [s]')
    end
end
fs('dInputs_travel')

%% display PID setpoint
figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on 
    if k == 1
        stairs(time, mpc_simple.Rplot(k, :).*kgcm2_to_pa, 'g')
        stairs(mpc_pid.tt, mpc_pid.u(k, :).*kgcm2_to_pa, 'k')
    else
        stairs(time, mpc_simple.Rplot(k, :), 'g')
        stairs(mpc_pid.tt, mpc_pid.u(k, :), 'k')
    end
    hold off
    ylabel(y_labels{k})
    if k == 1
        leg = legend('Ref', 'PID setpoint by MPC');
        set(leg, 'orientation', 'horizontal')
    end
    if k == 3
        xlabel('Time [s]')
    end
end
fs('PID_setpoints')