%% init
clear, close all

%% load data
extension = '_no_ref';
nom_model  = load('../boiler_ss_models/nominal_models.mat');
mpc_simple = load(['mpc_results_simple' extension]);
mpc_pid    = load(['mpc_results_pid' extension]);

%% extract data
y0     = mpc_simple.y0;
dumin  = mpc_simple.dumin;
dumax  = mpc_simple.dumax;
Ts     = mpc_simple.Ts;
us     = nom_model.us;
time   = mpc_simple.tt;
time_s = mpc_simple.y_pid(:, 1);

%% labels and other data
fpath = ['../thesis/images/' extension(2:end)];
papersize = [4.5, 2]*4;
y_labels = {'$p\;[MPa]$', '$P_0\;[MW]$', '$X_W\;[m]$'};
u_labels = {'$u_F\;[\%]$', '$u_S\;[\%]$', '$u_W\;[\%]$'};
du_labels = {'$\triangle u_F\;[\%/s]$', '$\triangle u_S\;[\%/s]$', ...
             '$\triangle u_W\;[\%/s]$'};
save_y = {'y1', 'y2', 'y3'};
save_u = {'u1', 'u2', 'u3'};
save_du = {'du1', 'du2', 'du3'};
save_tu = {'tu1', 'tu2', 'tu3'};
save_r = {'r1', 'r2', 'r3'};

%% data convert
kgcm2_to_pa = 98066.5e-6;
mpc_simple.y(1, :)     = mpc_simple.y(1, :).*kgcm2_to_pa;
mpc_pid.y(1+3, :)      = mpc_pid.y(1+3, :).*kgcm2_to_pa;
mpc_simple.y_pid(:, 2) = mpc_simple.y_pid(:, 2).*kgcm2_to_pa;

%% display Outputs
close all
for k = 1:1:3
    figure(k)
%     subplot(3, 1, k)
    hold on
    if k == 1
        stairs(time, (mpc_simple.Rplot(k, :) + y0(k)).*kgcm2_to_pa, 'g')
    else
        stairs(time, mpc_simple.Rplot(k, :) + y0(k), 'g')
    end
    plot(time_s, mpc_simple.y_pid(:, k+1), 'k')
    stairs(time, mpc_simple.y(k, :), 'b')
    stairs(mpc_pid.tt, mpc_pid.y(k+3, :), 'm')
    hold off
    ylabel(y_labels{k})
    if k == 1
        leg = legend('Ref', 'PID', 'MPC', 'MPC-PID');
%         set(leg, 'orientation', 'horizontal')
%         axis([0, 100, 10.59, 10.60])
%         set(gca, 'YLim', [10.59, 10.60], 'Ytick', [10.59, 10.60])
    end
%     if k == 3
        xlabel('Time [s]')
%     end
    set(gca, 'Xlim', [0 time(end)]);
    fsp(save_y{k}, save_y{k}, fpath, papersize)
end

%% display Inputs
close all
for k = 1:1:3
%     subplot(3, 1, k)
    figure(k)
    hold on
%     stairs(time, mpc_simple.Rplot(k, :) + y0(k), 'g')
    plot(mpc_simple.u_pid(:, 1), mpc_simple.u_pid(:, k+1) + us(k), 'k')
    stairs(time, mpc_simple.u(k, :) + us(k), 'b')
    stairs(mpc_pid.tt, mpc_pid.y(k, :) + us(k), 'm')
    stairs(mpc_pid.tt, mpc_pid.y(k, :)*0 + 1, 'r')
    stairs(mpc_pid.tt, mpc_pid.y(k, :)*0 + 0, 'r')
    hold off
    ylabel(u_labels{k})
    leg = legend('PID', 'MPC', 'MPC-PID');
%     set(leg, 'orientation', 'horizontal')
    if k == 3
        xlabel('Time [s]')
    end
%     if k == 2
%         set(gca, 'Xlim', [0 time(end)], 'Ylim', [-0.05, 1.05], 'Ytick', 0.3:0.1:0.9);
%     else
        set(gca, 'Xlim', [0 time(end)], 'Ylim', [-0.05, 1.05], 'Ytick', 0:0.2:1);
%     end
    fsp(save_u{k}, save_u{k}, fpath, papersize)
end

%% display delta Inputs
kk = gcf;
for k = 1:1:3
%     subplot(3, 1, k)
    figure(k+kk)
    hold on
    stairs(mpc_simple.u_pid(1:end-1, 1), diff(mpc_simple.u_pid(:, k+1)), 'k')
    stairs(time(1:end-1), diff(mpc_simple.u(k, :))/Ts, 'b')
    stairs(mpc_pid.tt(1:end-1), diff(mpc_pid.y(k, :))/Ts, 'm')
%     if k ~= 2
        stairs(time, mpc_simple.u(k, :)*0+dumin(k)/Ts, 'r')
%     end
    stairs(time, mpc_simple.u(k, :)*0+dumax(k)/Ts, 'r')
    hold off
    ylabel(du_labels{k})
    if k == 1
        leg = legend('PID', 'MPC', 'MPC-PID');
%         set(leg, 'orientation', 'horizontal')
    end
%     if k == 3
        xlabel('Time [s]')
%     end
    
    if k == 1
        axis([0 time(end) -0.025 0.025])
    elseif k == 2
        axis([0 time(end) -0.1 0.05])        
    elseif k == 3
        axis([0 time(end) -0.1 0.1])
    end
    set(gca, 'Xlim', [0 time(end)]);
    fsp(save_du{k}, save_du{k}, fpath, papersize)
end

%% display delta input travel
close all
for k = 1:1:3
%     subplot(3, 1, k)
    figure(k)
    hold on
    stairs(mpc_simple.u_pid(1:end-1, 1), cumsum(abs(diff(mpc_simple.u_pid(:, k+1)))), 'k')
    stairs(time(1:end-1), cumsum(abs(diff(mpc_simple.u(k, :))/Ts)), 'b')
    stairs(mpc_pid.tt(1:end-1), cumsum(abs(diff(mpc_pid.y(k, :))/Ts)), 'm')
    ylabel(u_labels{k})
    if k == 1
        leg = legend('PID', 'MPC', 'MPC-PID', 2);
%         set(leg, 'orientation', 'horizontal')
    end
%     if k == 3
        xlabel('Time [s]')
%     end
    hold off
    if k == 1 
        set(gca, 'Xlim', [0 time(end)], 'YLim', [-0.1, 2.5]);
    elseif k == 2
        set(gca, 'Xlim', [0 time(end)], 'YLim', [-0.1, 1.6]);
    else
        set(gca, 'Xlim', [0 time(end)], 'YLim', [-0.5, 7]);
    end
    fsp(save_tu{k}, save_tu{k}, fpath, papersize)
end
fs('dInputs_travel')

%% display PID setpoint
close all
for k = 1:1:3
    figure(k)
    hold on 
    if k == 1
        stairs(time, mpc_simple.Rplot(k, :).*kgcm2_to_pa, 'g')
        stairs(mpc_pid.tt, mpc_pid.u(k, :).*kgcm2_to_pa, 'm')
    else
        stairs(time, mpc_simple.Rplot(k, :), 'g')
        stairs(mpc_pid.tt, mpc_pid.u(k, :), 'm')
    end
    hold off
    ylabel(y_labels{k})
    if k == 1
        leg = legend('Ref', 'PID setpoint by MPC');
%         set(leg, 'orientation', 'horizontal')
    end
%     if k == 3
        xlabel('Time [s]')
%     end
    set(gca, 'Xlim', [0 time(end)]);
    fsp(save_r{k}, save_r{k}, fpath, papersize)
end
fs('PID_setpoints')
close all