%% init
clear, close all
Ts = 0.5;

%% load model
load('../boiler_ss_models/nominal_models');
load('controller_data.mat')
sysdBOILER = c2d(syscBOILER, Ts);
sysdCL = c2d(syscCL, Ts);
extension = '_no_ref';

ss_m.A = sysdCL.A;
ss_m.B = sysdCL.B;
ss_m.C = sysdCL.C;
ss_m.D = sysdCL.D;

ss_m.E = ss_m.B*0;
ss_m.F = ss_m.D*0;

%% MPC setup
N = 40;
weight.Q  = diag([0, 0, 0, 100, 0.01, 1]);
weight.S  = diag([1, 0.01, 1]);
weight.Sy = diag([0, 0, 0, 0, 0, 0]);
weight.Su = diag([0, 0, 0]);
us = [0.34; 0.69; 0.436];
ymin = [0; 0; 0; -inf; -inf; -inf] - [us; zeros(3, 1)];
ymax = [1; 1; 1; inf; inf; inf] - [us; zeros(3, 1)];
Ymin = repmat(ymin, N, 1);
Ymax = repmat(ymax, N, 1);

dymin = [-0.007; -2; -0.05; -inf; -inf; -inf]*Ts;
dymax = [0.007; 0.02; 0.05; inf; inf; inf]*Ts;
dYmin = repmat(dymin, N, 1);
dYmax = repmat(dymax, N, 1);
bdmin = dYmin;
bdmax = dYmax;

mpc_matrix = mpc_matrix_design(ss_m, weight, N);
H = mpc_matrix.H;
H1 = mpc_matrix.H1;
MX = mpc_matrix.Mx;
MR = mpc_matrix.Mr;
MU = mpc_matrix.Mu;
F3  = mpc_matrix.F3;
F4  = mpc_matrix.F4;
PHI = mpc_matrix.PHI;
PHIY = mpc_matrix.PHIY;
GM = mpc_matrix.GAMMA;
GMY = mpc_matrix.GAMMAY;

time = 400;
power_step_time = 100;
% run run_simulation
% sim('baseline_model_pid_mpc', [0, time]);
y0 = [0, 0, 0, ys];

%% setup simulation
Tmax = time/60; % in "min"
kf   = Tmax*60/Ts;
t    = 0:Ts:kf;
tt   = linspace(0, Tmax*60, kf);

x = zeros(size(ss_m.A, 1), kf+1);
xp = zeros(size(ss_m.A, 1), kf+1);
y = zeros(6, kf);
yp = zeros(6, kf);
u = zeros(3, kf);
u_qp = [0, 0, 0]';
nu = 3;
ny = 6;
reference.init  = [0; 0; 0; 0; 0; 0];
reference.value = [0; 0; 0; 0; 5; 0];
reference.start = [0; 0; 0; 0; 25/Ts; 0];
reference.end   = [kf+N, kf+N, kf+N, kf+N, kf+N, kf+N];
[R, Rplot] = reference_build_wave(N, kf, reference);
y_temp = y0'*0;

%% run simulation
tic;
display('----------------------------')
display('Entering the simulation loop')
fprintf(1,'Status:     ');
for k = 1:1:kf
    % for-loop diagnostic
    for_diag(k, kf)
    
    % MPC controller
%     dR = cell2mat(R(k:k+N-1));
    dR = repmat(cell2mat(R(k)), N, 1);

    % Constraints    
    x_temp = PHIY*x(:, k);
    bdmin = dYmin - x_temp;
    bdmax = dYmax - x_temp;
    bdmin(1:ny) = bdmin(1:ny) + y_temp;
    bdmax(1:ny) = bdmax(1:ny) + y_temp;
    
    bmax = Ymax - PHI*x(:, k);
    bmin = Ymin - PHI*x(:, k);
    b    = [bdmax; -bdmin; bmax; -bmin];
%     b    = [bdmax; -bdmin];
%     b = [bdmax; -bdmin];
    
    g  = MX*x(:, k) + MR*dR + MU*u_qp;
    [U, err] = qpas(H, g, F4, b, [], [], [], []);
%     U = qpas(H, g, [], [], [], [], [], []);

    if err ~= 0
        return
    end
    
    u_qp = U(1:nu);
    u_in = u_qp;
    
    x(:, k+1) = ss_m.A*x(:, k) + ss_m.B*u_in;
    y_temp    = ss_m.C*x(:, k) + ss_m.D*u_in;
    y(:, k) = y_temp + y0';
    u(:, k) = u_in;
    
    
    % PID simulation
    xp(:, k+1) = ss_m.A*xp(:, k) + ss_m.B*Rplot(4:6, k);
    yp(:, k)   = ss_m.C*xp(:, k) + ss_m.D*Rplot(4:6, k);
end
fprintf('\n')
time = toc;
cprintf([0.0, 0.0, 0.8], 'Simulation complete\n');
display(['Elapsed time: ' num2str(time) ' s'])
display(['Samples per second: ' num2str(kf/time)])

%% save data
save(['mpc_results_pid' extension], 'Rplot', 'y', 'u', 'yp', 'tt', 'y0',...
     'dymin', 'dymax', 'Ts');

% run display_data

%% Display results
close all
display('Displaying results')
figure
subplot(3, 1, 1)
    hold on
    stairs(tt, Rplot(1+3, :) + y0(1+3), 'g')
    stairs(tt, y(1+3, :))
    stairs(tt, yp(4, :) + y0(1+3), 'm')
    hold off
    ylabel('Pressure')
subplot(3, 1, 2)
    hold on
    stairs(tt, Rplot(2+3, :) + y0(2+3), 'g')
    stairs(tt, y(2+3, :))
    stairs(tt, yp(5, :) + y0(2+3), 'm')
    hold off
    ylabel('Power')
subplot(3, 1, 3)
    hold on
    stairs(tt, Rplot(3+3, :) + y0(3+3), 'g')
    stairs(tt, y(3+3, :))
    stairs(tt, yp(6, :) + y0(3+3), 'm')
    hold off
    ylabel('Xw')
fs('Output')

figure
subplot(3, 1, 1)
    hold on
    stairs(tt, tt*0 + 0, 'r')
    stairs(tt, tt*0 + 1, 'r')
    stairs(tt, y(1, :) + us(1), 'b')
    stairs(tt, yp(1, :) + us(1), 'm')
    hold off
    ylabel('Fuel')
subplot(3, 1, 2)
    hold on
    stairs(tt, tt*0 + 0, 'r')
    stairs(tt, tt*0 + 1, 'r')
    stairs(tt, y(2, :) + us(2), 'b')
    stairs(tt, yp(2, :) + us(2), 'm')
    hold off
    ylabel('Steam')
subplot(3, 1, 3)
    hold on
    stairs(tt, tt*0 + 0, 'r')
    stairs(tt, tt*0 + 1, 'r')
    stairs(tt, y(3, :) + us(3), 'b')
    stairs(tt, yp(3, :) + us(3), 'm')
    hold off
    ylabel('Feedwater')
fs('Input')

figure
subplot(3, 1, 1)
    hold on
    stairs(tt(1:end-1), diff(y(1, :))/Ts, 'b')
    stairs(tt(1:end-1), diff(yp(1, :))/Ts, 'm')
    stairs(tt(1:end-1), diff(y(1, :))*0+dymin(1)/Ts, 'r')
    stairs(tt(1:end-1), diff(y(1, :))*0+dymax(1)/Ts, 'r')
    hold off
    ylabel('Fuel')
subplot(3, 1, 2)
    hold on
    stairs(tt(1:end-1), diff(y(2, :))/Ts, 'b')
    stairs(tt(1:end-1), diff(yp(2, :))/Ts, 'm')
%     stairs(tt(1:end-1), diff(y(2, :))*0+dymin(2)/Ts, 'r')
    stairs(tt(1:end-1), diff(y(2, :))*0+dymax(2)/Ts, 'r')
    hold off
    ylabel('Steam')
subplot(3, 1, 3)
    hold on
    stairs(tt(1:end-1), diff(y(3, :))/Ts, 'b')
    stairs(tt(1:end-1), diff(yp(3, :))/Ts, 'm')
    stairs(tt(1:end-1), diff(y(3, :))*0+dymin(3)/Ts, 'r')
    stairs(tt(1:end-1), diff(y(3, :))*0+dymax(3)/Ts, 'r')
    hold off
    ylabel('Feedwater')
fs('dInput')