%% init
clear, close all

%% load data
load('../boiler_ss_models/nominal_models.mat')

%% set up simulation
Ts = 1;
time = 200/60;
Tmax = time; % in "min"
kf   = Tmax*60/Ts;
t    = 0:Ts:kf;
tt   = linspace(0, Tmax*60, kf);
ss_m = c2d(syscBOILER, Ts);

x = zeros(size(ss_m.A, 1), kf);
y = zeros(3, kf);
u = zeros(3, kf);
X_in = xs;

u(1, ceil(5):end) = 0.1;
meas = cell(kf, 1);

options_ode = odeset('MaxStep', Ts/2, 'MaxOrder', 3, 'AbsTol', 1e-5);

%% run simulation
tic;
display('----------------------------')
display('Entering the simulation loop')
fprintf(1,'Status:     ');
for k = 1:1:kf
    % for-loop diagnostic
    for_diag(k, kf)
    
    u_in = u(:, k);
    x(:, k+1) = ss_m.A*x(:, k) + ss_m.B*u_in;
    y(:, k)   = ss_m.C*x(:, k) + ss_m.D*u_in + ys';
    
    uo = (u_in + us)';
    [~, X] = ode45(@bnl,[t(k), t(k)+Ts], X_in, options_ode, uo);
    X_in = X(end, :)';
    qe  = (0.854*uo(2) - 0.147)*X(:, 1) + 45.59*uo(1) - 2.514*uo(3) - 2.096;
    acs = ( (1-0.001538*X(:, 3)) .* (0.8*X(:, 1) - 25.6) ) ./...
          ( X(3).*(1.0394 - 0.0012304.*X(:, 1)) );
    Xw  = 0.05*(0.13073.*X(:, 3) + 100.*acs + qe./9 - 67.975);
%     acs =  
    meas(k, 1) = {[X, Xw]};
end
fprintf('\n')
time = toc;
cprintf([0.0, 0.0, 0.8], 'Simulation complete\n');
display(['Elapsed time: ' num2str(time) ' s'])
display(['Samples per second: ' num2str(kf/time)])

%% display data
meas = cell2mat(meas);
tode = linspace(0, k/kf*Tmax*60, length(meas));
figure
for k = 1:1:3
    subplot(3, 1, k)
    hold on
    stairs(tt, y(k, :))
    if k == 3
        plot(tode, meas(:, k+1), 'm')
    else
        plot(tode, meas(:, k), 'm')
    end
    hold off
end